/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package impl.message.AllDataType;

import java.nio.BufferUnderflowException;

import kr.pe.sinnori.common.converter.AbstractB2IConverter;
import kr.pe.sinnori.common.exception.BodyFormatException;
import kr.pe.sinnori.common.io.StreamReaderIF;
import kr.pe.sinnori.common.message.input.AbstractInputMessage;

/**
 * DHB 방식으로 이진 스트림을 AllDataType 입력 메시지로 변환하는 변환기
 * 
 * @author Won Jonghoon
 * 
 */
public final class AllDataTypeB2IDHB extends AbstractB2IConverter {
	@Override
	protected AbstractInputMessage doTask(StreamReaderIF sr)
			throws BufferUnderflowException, IllegalArgumentException,
			BodyFormatException {
		AllDataTypeInputMessage inObj = new AllDataTypeInputMessage();
		inObj.byteVar1 = sr.getByte();
		inObj.byteVar2 = sr.getByte();
		inObj.byteVar3 = sr.getByte();

		inObj.unsignedByteVar1 = sr.getUnsignedByte();
		inObj.unsignedByteVar2 = sr.getUnsignedByte();
		inObj.unsignedByteVar3 = sr.getUnsignedByte();

		inObj.shortVar1 = sr.getShort();
		inObj.shortVar2 = sr.getShort();
		inObj.shortVar3 = sr.getShort();

		inObj.unsignedShortVar1 = sr.getUnsignedShort();
		inObj.unsignedShortVar2 = sr.getUnsignedShort();
		inObj.unsignedShortVar3 = sr.getUnsignedShort();
		inObj.intVar1 = sr.getInt();
		inObj.intVar2 = sr.getInt();
		inObj.intVar3 = sr.getInt();
		inObj.unsignedIntVar1 = sr.getUnsignedInt();
		inObj.unsignedIntVar2 = sr.getUnsignedInt();
		inObj.unsignedIntVar3 = sr.getUnsignedInt();
		inObj.longVar1 = sr.getLong();
		inObj.longVar2 = sr.getLong();
		inObj.longVar3 = sr.getLong();

		inObj.strVar1 = sr.getSIPascalString().trim();
		inObj.strVar2 = sr.getUBPascalString().trim();

		inObj.strVar3 = sr.getUSPascalString().trim();
		inObj.bytesVar1 = sr.getBytes(2);
		inObj.cnt = sr.getUnsignedShort();
		inObj.mebmerList = new AllDataTypeInputMessage.Member[inObj.cnt];
		// int memeber_cnt = one_out.mebmerList.length;
		for (int i = 0; i < inObj.cnt; i++) {
			inObj.mebmerList[i] = inObj.new Member();
			inObj.mebmerList[i].memberID = sr.getString(30).trim();
			inObj.mebmerList[i].memberName = sr.getString(30).trim();

			inObj.mebmerList[i].cnt = sr.getUnsignedShort();

			inObj.mebmerList[i].itemList = new AllDataTypeInputMessage.Member.Item[inObj.mebmerList[i].cnt];
			for (int j = 0; j < inObj.mebmerList[i].cnt; j++) {
				inObj.mebmerList[i].itemList[j] = inObj.mebmerList[i].new Item();
				inObj.mebmerList[i].itemList[j].itemID = sr.getString(30)
						.trim();
				inObj.mebmerList[i].itemList[j].itemName = sr.getString(30)
						.trim();
				inObj.mebmerList[i].itemList[j].itemCnt = sr.getUnsignedShort();
			}
		}

		return inObj;
	}
}
