/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package impl.message.AllDataType;

import java.nio.BufferUnderflowException;

import kr.pe.sinnori.common.converter.AbstractB2OConverter;
import kr.pe.sinnori.common.exception.BodyFormatException;
import kr.pe.sinnori.common.io.StreamReaderIF;
import kr.pe.sinnori.common.message.output.AbstractOutputMessage;

/**
 * DHB 방식으로 이진 스트림을 BigByte 출력 메시지로 변환하는 변환기
 * 
 * @author Won Jonghoon
 * 
 */
public final class AllDataTypeB2ODHB extends AbstractB2OConverter {
	@Override
	public AbstractOutputMessage doTask(StreamReaderIF sr)
			throws BufferUnderflowException, IllegalArgumentException,
			BodyFormatException {
		AllDataTypeOutputMessage outObj = new AllDataTypeOutputMessage();
		outObj.byteVar1 = sr.getByte();
		outObj.byteVar2 = sr.getByte();
		outObj.byteVar3 = sr.getByte();

		outObj.unsignedByteVar1 = sr.getUnsignedByte();
		outObj.unsignedByteVar2 = sr.getUnsignedByte();
		outObj.unsignedByteVar3 = sr.getUnsignedByte();

		outObj.shortVar1 = sr.getShort();
		outObj.shortVar2 = sr.getShort();
		outObj.shortVar3 = sr.getShort();
		outObj.unsignedShortVar1 = sr.getUnsignedShort();
		outObj.unsignedShortVar2 = sr.getUnsignedShort();
		outObj.unsignedShortVar3 = sr.getUnsignedShort();

		outObj.intVar1 = sr.getInt();
		outObj.intVar2 = sr.getInt();
		outObj.intVar3 = sr.getInt();

		outObj.unsignedIntVar1 = sr.getUnsignedInt();
		outObj.unsignedIntVar2 = sr.getUnsignedInt();
		outObj.unsignedIntVar3 = sr.getUnsignedInt();

		outObj.longVar1 = sr.getLong();
		outObj.longVar2 = sr.getLong();
		outObj.longVar3 = sr.getLong();
		outObj.strVar1 = sr.getSIPascalString();
		outObj.strVar2 = sr.getUBPascalString();
		outObj.strVar3 = sr.getUSPascalString();

		outObj.bytesVar1 = sr.getBytes(2);

		outObj.cnt = sr.getUnsignedShort();
		outObj.mebmerList = new AllDataTypeOutputMessage.Member[outObj.cnt];
		// int memeber_cnt = one_out.mebmerList.length;
		for (int i = 0; i < outObj.cnt; i++) {
			outObj.mebmerList[i] = outObj.new Member();
			outObj.mebmerList[i].memberID = sr.getString(30).trim();
			outObj.mebmerList[i].memberName = sr.getString(30).trim();
			outObj.mebmerList[i].cnt = sr.getUnsignedShort();

			outObj.mebmerList[i].itemList = new AllDataTypeOutputMessage.Member.Item[outObj.mebmerList[i].cnt];
			for (int j = 0; j < outObj.mebmerList[i].cnt; j++) {
				outObj.mebmerList[i].itemList[j] = outObj.mebmerList[i].new Item();
				outObj.mebmerList[i].itemList[j].itemID = sr.getString(30)
						.trim();
				outObj.mebmerList[i].itemList[j].itemName = sr.getString(30)
						.trim();
				outObj.mebmerList[i].itemList[j].itemCnt = sr
						.getUnsignedShort();
			}
		}

		return outObj;
	}
}
