/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package impl.message.AllDataType;

import java.nio.BufferOverflowException;

import kr.pe.sinnori.common.converter.AbstractI2BConverter;
import kr.pe.sinnori.common.exception.NotEnoughBodyBufferException;
import kr.pe.sinnori.common.io.StreamWriterIF;
import kr.pe.sinnori.common.message.input.AbstractInputMessage;

/**
 * DHB 방식으로 AllDataType 입력 메시지를 이진 스트림으로 변환하는 변환기
 * 
 * @author Won Jonghoon
 * 
 */
public final class AllDataTypeI2BDHB extends AbstractI2BConverter {

	@Override
	protected void doTask(AbstractInputMessage inputMessage, StreamWriterIF cos)
			throws BufferOverflowException, IllegalArgumentException,
			NotEnoughBodyBufferException {
		AllDataTypeInputMessage inObj = (AllDataTypeInputMessage) inputMessage;

		cos.putByte(inObj.byteVar1);
		cos.putByte(inObj.byteVar2);
		cos.putByte(inObj.byteVar3);

		cos.putUnsignedByte(inObj.unsignedByteVar1);
		cos.putUnsignedByte(inObj.unsignedByteVar2);
		cos.putUnsignedByte(inObj.unsignedByteVar3);

		cos.putShort(inObj.shortVar1);
		cos.putShort(inObj.shortVar2);
		cos.putShort(inObj.shortVar3);
		cos.putUnsignedShort(inObj.unsignedShortVar1);
		cos.putUnsignedShort(inObj.unsignedShortVar2);
		cos.putUnsignedShort(inObj.unsignedShortVar3);
		cos.putInt(inObj.intVar1);
		cos.putInt(inObj.intVar2);
		cos.putInt(inObj.intVar3);
		cos.putUnsignedInt(inObj.unsignedIntVar1);
		cos.putUnsignedInt(inObj.unsignedIntVar2);
		cos.putUnsignedInt(inObj.unsignedIntVar3);
		cos.putLong(inObj.longVar1);
		cos.putLong(inObj.longVar2);
		cos.putLong(inObj.longVar3);
		cos.putSIPascalString(inObj.strVar1);
		cos.putUBPascalString(inObj.strVar2);
		cos.putUSPascalString(inObj.strVar3);
		cos.putBytes(inObj.bytesVar1);
		cos.putUnsignedShort(inObj.cnt);

		if (inObj.cnt < 0) {
			throw new IllegalArgumentException(String.format(
					"'멤버 횟수[%d]는 0보다 크거나 같아야 합니다.", inObj.cnt));
		}

		if (null != inObj.mebmerList) {
			for (int i = 0; i < inObj.cnt; i++) {
				if (inObj.mebmerList[i].cnt < 0) {
					throw new IllegalArgumentException(String.format(
							"'아이템 갯수[%d]는 0보다 크거나 같아야 합니다.",
							inObj.mebmerList[i].cnt));
				}

				cos.putString(30, inObj.mebmerList[i].memberID);
				cos.putString(30, inObj.mebmerList[i].memberName);
				cos.putUnsignedShort(inObj.mebmerList[i].cnt);

				if (null != inObj.mebmerList[i].itemList) {
					for (int j = 0; j < inObj.mebmerList[i].cnt; j++) {
						cos.putString(30,
								inObj.mebmerList[i].itemList[j].itemID);
						cos.putString(30,
								inObj.mebmerList[i].itemList[j].itemName);
						cos.putUnsignedShort(inObj.mebmerList[i].itemList[j].itemCnt);
					}
				} else {
					if (inObj.mebmerList[i].cnt >= 0) {
						throw new IllegalArgumentException(String.format(
								"아이템 갯수[%d] 있지만 itemList 가 null입니다.",
								inObj.mebmerList[i].cnt));
					}
				}
			}
		} else {
			if (inObj.cnt >= 0) {
				throw new IllegalArgumentException(String.format(
						"'멤버 횟수[%d] 있지만 mebmerList 가 null입니다.", inObj.cnt));
			}
		}

	}

}
