/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package impl.message.AllDataType;

import java.nio.BufferOverflowException;

import kr.pe.sinnori.common.converter.AbstractO2BConverter;
import kr.pe.sinnori.common.exception.NotEnoughBodyBufferException;
import kr.pe.sinnori.common.io.StreamWriterIF;
import kr.pe.sinnori.common.message.output.AbstractOutputMessage;

/**
 * DHB 방식으로 AllDataType 출력 메시지를 이진 스트림으로 변환하는 변환기
 * 
 * @author Won Jonghoon
 * 
 */
public final class AllDataTypeO2BDHB extends AbstractO2BConverter {
	@Override
	protected void doTask(AbstractOutputMessage outputMessage, StreamWriterIF sw)
			throws BufferOverflowException, IllegalArgumentException,
			NotEnoughBodyBufferException {
		AllDataTypeOutputMessage outObj = (AllDataTypeOutputMessage) outputMessage;

		sw.putByte(outObj.byteVar1);
		sw.putByte(outObj.byteVar2);
		sw.putByte(outObj.byteVar3);

		sw.putUnsignedByte(outObj.unsignedByteVar1);
		sw.putUnsignedByte(outObj.unsignedByteVar2);
		sw.putUnsignedByte(outObj.unsignedByteVar3);

		sw.putShort(outObj.shortVar1);
		sw.putShort(outObj.shortVar2);
		sw.putShort(outObj.shortVar3);

		sw.putUnsignedShort(outObj.unsignedShortVar1);
		sw.putUnsignedShort(outObj.unsignedShortVar2);
		sw.putUnsignedShort(outObj.unsignedShortVar3);

		sw.putInt(outObj.intVar1);
		sw.putInt(outObj.intVar2);
		sw.putInt(outObj.intVar3);
		sw.putUnsignedInt(outObj.unsignedIntVar1);
		sw.putUnsignedInt(outObj.unsignedIntVar2);
		sw.putUnsignedInt(outObj.unsignedIntVar3);
		sw.putLong(outObj.longVar1);
		sw.putLong(outObj.longVar2);
		sw.putLong(outObj.longVar3);
		sw.putSIPascalString(outObj.strVar1);
		sw.putUBPascalString(outObj.strVar2);
		sw.putUSPascalString(outObj.strVar3);
		sw.putBytes(outObj.bytesVar1);
		sw.putUnsignedShort(outObj.cnt);

		if (outObj.cnt < 0) {
			throw new IllegalArgumentException(String.format(
					"'멤버 횟수[%d]는 0보다 크거나 같아야 합니다.", outObj.cnt));
		}

		if (null != outObj.mebmerList) {
			for (int i = 0; i < outObj.cnt; i++) {
				sw.putString(30, outObj.mebmerList[i].memberID);
				sw.putString(30, outObj.mebmerList[i].memberName);
				sw.putUnsignedShort(outObj.mebmerList[i].cnt);

				if (null != outObj.mebmerList[i].itemList) {
					for (int j = 0; j < outObj.mebmerList[i].cnt; j++) {
						sw.putString(30,
								outObj.mebmerList[i].itemList[j].itemID);
						sw.putString(30,
								outObj.mebmerList[i].itemList[j].itemName);
						sw.putUnsignedShort(outObj.mebmerList[i].itemList[j].itemCnt);
					}
				} else {
					if (outObj.mebmerList[i].cnt > 0) {
						throw new IllegalArgumentException(String.format(
								"'아이템 갯수[%d] 있지만 itemList 가 null입니다.",
								outObj.mebmerList[i].cnt));
					}
				}
			}
		} else {
			if (outObj.cnt > 0) {
				throw new IllegalArgumentException(String.format(
						"'멤버 횟수[%d] 있지만 mebmerList 가 null입니다.", outObj.cnt));
			}
		}
		// sw.skip(6);
	}
}
