/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package impl.message.AllDataType;

import java.nio.channels.SocketChannel;

import kr.pe.sinnori.common.message.input.AbstractInputMessage;
import kr.pe.sinnori.server.executor.AbstractServerExecutor;
import kr.pe.sinnori.server.io.LetterListToClient;

/**
 * 메세지 식별자 AllDataType 에 대응하는 실행기
 * 
 * @author Won Jonghoon
 * 
 */
public final class AllDataTypeServerExecutor extends AbstractServerExecutor {

	@Override
	protected void doTask(SocketChannel fromSC,
			AbstractInputMessage inputMessage,
			LetterListToClient letterToClientList) {
		AllDataTypeInputMessage inObj = (AllDataTypeInputMessage) inputMessage;

		AllDataTypeOutputMessage outObj = new AllDataTypeOutputMessage();
		outObj.byteVar1 = inObj.byteVar1;
		outObj.byteVar2 = inObj.byteVar2;
		outObj.byteVar3 = inObj.byteVar3;

		outObj.unsignedByteVar1 = inObj.unsignedByteVar1;
		outObj.unsignedByteVar2 = inObj.unsignedByteVar2;
		outObj.unsignedByteVar3 = inObj.unsignedByteVar3;

		outObj.shortVar1 = inObj.shortVar1;
		outObj.shortVar2 = inObj.shortVar2;
		outObj.shortVar3 = inObj.shortVar3;
		outObj.unsignedShortVar1 = inObj.unsignedShortVar1;
		outObj.unsignedShortVar2 = inObj.unsignedShortVar2;
		outObj.unsignedShortVar3 = inObj.unsignedShortVar3;
		outObj.intVar1 = inObj.intVar1;
		outObj.intVar2 = inObj.intVar2;
		outObj.intVar3 = inObj.intVar3;
		outObj.unsignedIntVar1 = inObj.unsignedIntVar1;
		outObj.unsignedIntVar2 = inObj.unsignedIntVar2;
		outObj.unsignedIntVar3 = inObj.unsignedIntVar3;
		outObj.longVar1 = inObj.longVar1;
		outObj.longVar2 = inObj.longVar2;
		outObj.longVar3 = inObj.longVar3;
		outObj.strVar1 = inObj.strVar1;
		outObj.strVar2 = inObj.strVar2;
		outObj.strVar3 = inObj.strVar3;
		outObj.bytesVar1 = inObj.bytesVar1;

		outObj.cnt = inObj.cnt;

		outObj.mebmerList = new AllDataTypeOutputMessage.Member[outObj.cnt];

		for (int i = 0; i < outObj.cnt; i++) {
			outObj.mebmerList[i] = outObj.new Member();

			outObj.mebmerList[i].memberID = inObj.mebmerList[i].memberID;
			outObj.mebmerList[i].memberName = inObj.mebmerList[i].memberName;

			outObj.mebmerList[i].cnt = inObj.mebmerList[i].cnt;
			outObj.mebmerList[i].itemList = new AllDataTypeOutputMessage.Member.Item[outObj.mebmerList[i].cnt];

			for (int j = 0; j < outObj.mebmerList[i].cnt; j++) {
				outObj.mebmerList[i].itemList[j] = outObj.mebmerList[i].new Item();

				outObj.mebmerList[i].itemList[j].itemID = inObj.mebmerList[i].itemList[j].itemID;
				outObj.mebmerList[i].itemList[j].itemName = inObj.mebmerList[i].itemList[j].itemName;
				outObj.mebmerList[i].itemList[j].itemCnt = inObj.mebmerList[i].itemList[j].itemCnt;
			}
		}

		letterToClientList.addLetterToClient(fromSC, outObj);

	}
}
